#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Flask Quorum
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Flask Quorum.
#
# Hive Flask Quorum is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Flask Quorum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Flask Quorum. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import json

import quorum

class ExportTest(quorum.TestCase):

    def setUp(self):
        try:
            quorum.load(name = __name__)
        except:
            self.skip()

    def tearDown(self):
        try:
            adapter = quorum.get_adapter()
            adapter.drop_db()
        except: pass
        finally: quorum.unload()

    @quorum.secured
    def test_import_single(self):
        structure = {
            "person:id" : dict(_id = "person:id", seq = 11),
            "account:id" : dict(_id = "account:id", seq = 33)
        }
        data = json.dumps(structure)
        data = quorum.legacy.bytes(data)

        adapter = quorum.get_adapter()
        manager = quorum.export.ExportManager(
            adapter,
            multiple = quorum.resolve()
        )

        collection = adapter.collection("counter")
        manager._import_single(collection, data, key = "_id")

        values = collection.find()
        values = [value for value in values]

        self.assertEqual(type(values), list)
        self.assertEqual(len(values), 2)

        value = collection.find_one(dict(_id = "person:id"))

        self.assertEqual(value["seq"], 11)

    @quorum.secured
    def test_import_multiple(self):
        data = [
            (
                "person:id",
                quorum.legacy.bytes(
                    json.dumps(dict(_id = "person:id", seq = 11)),
                    encoding = "utf-8"
                )
            ),
            (
                "account:id",
                quorum.legacy.bytes(
                    json.dumps(dict(_id = "account:id", seq = 33)),
                    encoding = "utf-8"
                )
            )
        ]

        adapter = quorum.get_adapter()
        manager = quorum.export.ExportManager(
            adapter,
            multiple = quorum.resolve()
        )

        collection = adapter.collection("counter")
        manager._import_multiple(collection, data, key = "_id")

        values = collection.find()
        values = [value for value in values]

        self.assertEqual(type(values), list)
        self.assertEqual(len(values), 2)

        value = collection.find_one(dict(_id = "person:id"))

        self.assertEqual(value["seq"], 11)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Flask Quorum
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Flask Quorum.
#
# Hive Flask Quorum is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Flask Quorum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Flask Quorum. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import quorum

class ConfigTest(quorum.TestCase):

    @quorum.secured
    def test_basic(self):
        quorum.confs("NAME", "name")
        result = quorum.conf("NAME")

        self.assertEqual(result, "name")

        result = quorum.conf("NAME", cast = str)

        self.assertEqual(result, "name")
        self.assertEqual(type(result), str)

        result = quorum.conf("NAME", cast = "str")

        self.assertEqual(result, "name")
        self.assertEqual(type(result), str)

        quorum.confs("AGE", "10")
        result = quorum.conf("AGE", cast = int)

        self.assertEqual(result, 10)
        self.assertEqual(type(result), int)

        result = quorum.conf("AGE", cast = "int")

        self.assertEqual(result, 10)
        self.assertEqual(type(result), int)

        result = quorum.conf("AGE", cast = str)

        self.assertEqual(result, "10")
        self.assertEqual(type(result), str)

        result = quorum.conf("HEIGHT")

        self.assertEqual(result, None)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Flask Quorum
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Flask Quorum.
#
# Hive Flask Quorum is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Flask Quorum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Flask Quorum. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import calendar
import datetime

import quorum

class ExecutionTest(quorum.TestCase):

    @quorum.secured
    def test_seconds_eval(self):
        now = datetime.datetime(year = 2014, month = 1, day = 1, second = 0)
        result = quorum.seconds_eval(1, now = now)
        expected_d = datetime.datetime(year = 2014, month = 1, day = 1, second = 1)
        expected_t = calendar.timegm(expected_d.utctimetuple())
        self.assertEqual(result, expected_t)

        now = datetime.datetime(year = 2014, month = 1, day = 1, second = 59)
        result = quorum.seconds_eval(1, now = now)
        expected_d = datetime.datetime(year = 2014, month = 1, day = 1, minute = 1, second = 0)
        expected_t = calendar.timegm(expected_d.utctimetuple())
        self.assertEqual(result, expected_t)

    @quorum.secured
    def test_daily_eval(self):
        now = datetime.datetime(year = 2014, month = 1, day = 1)
        result = quorum.daily_eval(0, now = now)
        expected_d = datetime.datetime(year = 2014, month = 1, day = 2)
        expected_t = calendar.timegm(expected_d.utctimetuple())
        self.assertEqual(result, expected_t)

        now = datetime.datetime(year = 2013, month = 12, day = 31)
        result = quorum.daily_eval(0, now = now)
        expected_d = datetime.datetime(year = 2014, month = 1, day = 1)
        expected_t = calendar.timegm(expected_d.utctimetuple())
        self.assertEqual(result, expected_t)

        now = datetime.datetime(year = 2013, month = 12, day = 31, hour = 1)
        result = quorum.daily_eval(0, now = now)
        expected_d = datetime.datetime(year = 2014, month = 1, day = 1)
        expected_t = calendar.timegm(expected_d.utctimetuple())
        self.assertEqual(result, expected_t)

        now = datetime.datetime(year = 2013, month = 12, day = 31, hour = 1)
        result = quorum.daily_eval(3600, now = now)
        expected_d = datetime.datetime(year = 2014, month = 1, day = 1, hour = 1)
        expected_t = calendar.timegm(expected_d.utctimetuple())
        self.assertEqual(result, expected_t)

    @quorum.secured
    def test_weekly_eval(self):
        now = datetime.datetime(year = 2014, month = 1, day = 1)
        result = quorum.weekly_eval(4, 0, now = now)
        expected_d = datetime.datetime(year = 2014, month = 1, day = 3)
        expected_t = calendar.timegm(expected_d.utctimetuple())
        self.assertEqual(result, expected_t)

        now = datetime.datetime(year = 2014, month = 1, day = 6)
        result = quorum.weekly_eval(4, 0, now = now)
        expected_d = datetime.datetime(year = 2014, month = 1, day = 10)
        expected_t = calendar.timegm(expected_d.utctimetuple())
        self.assertEqual(result, expected_t)

        now = datetime.datetime(year = 2014, month = 1, day = 10, hour = 1)
        result = quorum.weekly_eval(4, 3600, now = now)
        expected_d = datetime.datetime(year = 2014, month = 1, day = 10, hour = 1)
        expected_t = calendar.timegm(expected_d.utctimetuple())
        self.assertEqual(result, expected_t)

        now = datetime.datetime(year = 2014, month = 1, day = 10, hour = 3)
        result = quorum.weekly_eval(4, 3600, now = now)
        expected_d = datetime.datetime(year = 2014, month = 1, day = 17, hour = 1)
        expected_t = calendar.timegm(expected_d.utctimetuple())
        self.assertEqual(result, expected_t)

        now = datetime.datetime(year = 2014, month = 1, day = 11, hour = 3)
        result = quorum.weekly_eval(4, 3600, now = now)
        expected_d = datetime.datetime(year = 2014, month = 1, day = 17, hour = 1)
        expected_t = calendar.timegm(expected_d.utctimetuple())
        self.assertEqual(result, expected_t)

    @quorum.secured
    def test_monthly_eval(self):
        now = datetime.datetime(year = 2014, month = 1, day = 1)
        result = quorum.monthly_eval(6, 0, now = now)
        expected_d = datetime.datetime(year = 2014, month = 1, day = 6)
        expected_t = calendar.timegm(expected_d.utctimetuple())
        self.assertEqual(result, expected_t)

        now = datetime.datetime(year = 2014, month = 1, day = 6)
        result = quorum.monthly_eval(6, 0, now = now)
        expected_d = datetime.datetime(year = 2014, month = 1, day = 6)
        expected_t = calendar.timegm(expected_d.utctimetuple())
        self.assertEqual(result, expected_t)

        now = datetime.datetime(year = 2014, month = 1, day = 6, hour = 1)
        result = quorum.monthly_eval(6, 0, now = now)
        expected_d = datetime.datetime(year = 2014, month = 2, day = 6)
        expected_t = calendar.timegm(expected_d.utctimetuple())
        self.assertEqual(result, expected_t)

        now = datetime.datetime(year = 2014, month = 1, day = 6, hour = 1)
        result = quorum.monthly_eval(6, 3600, now = now)
        expected_d = datetime.datetime(year = 2014, month = 1, day = 6, hour = 1)
        expected_t = calendar.timegm(expected_d.utctimetuple())
        self.assertEqual(result, expected_t)

        now = datetime.datetime(year = 2014, month = 1, day = 6, hour = 2)
        result = quorum.monthly_eval(6, 3600, now = now)
        expected_d = datetime.datetime(year = 2014, month = 2, day = 6, hour = 1)
        expected_t = calendar.timegm(expected_d.utctimetuple())
        self.assertEqual(result, expected_t)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Flask Quorum
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Flask Quorum.
#
# Hive Flask Quorum is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Flask Quorum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Flask Quorum. If not, see <http://www.apache.org/licenses/>.

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

from . import acl
from . import amazon
from . import amqp
from . import base
from . import config
from . import crypt
from . import daemon
from . import data
from . import defines
from . import errors
from . import exceptions
from . import execution
from . import export
from . import extras
from . import formats
from . import httpc
from . import jsonf
from . import legacy
from . import log
from . import mail
from . import meta
from . import model
from . import mongodb
from . import observer
from . import pusherc
from . import redisdb
from . import request
from . import route
from . import session
from . import storage
from . import structures
from . import template
from . import typesf
from . import unit_test
from . import util
from . import validation

from . import info as _info

from .acl import check_basic_auth, check_login, check_token, check_tokens, ensure_basic_auth,\
    ensure_login, ensure_user, ensure_session, ensure, ensure_auth, get_tokens_m, to_tokens_m
from .base import APP, RUN_CALLED, RUN_F, Quorum, monkey, call_run, run, prepare_app,\
    run_base, run_waitress, run_netius, load, unload, load_all, load_app_config,\
    load_paths, load_bundles, start_log, extra_logging, get_app, get_adapter, get_log,\
    get_level, get_handlers, get_handler, get_bundle, is_devel, finalize, before_request,\
    after_request, context_processor, start_execution, stop_execution, setup_models,\
    models_c, resolve, templates_path, bundles_path, base_path, has_context, ensure_context, onrun
from .config import conf, conf_prefix, conf_suffix, confs, confr, confd
from .crypt import Cipher, RC4, Spritz
from .daemon import Daemon
from .data import DataAdapter, MongoAdapter, TinyAdapter, Collection, MongoCollection, TinyCollection
from .defines import ITERABLES, MOBILE_REGEX, TABLET_REGEX, MOBILE_PREFIX_REGEX, BROWSER_INFO, OS_INFO,\
    WINDOWS_LOCALE
from .errors import errors_json
from .exceptions import BaseError, ServerInitError, ModuleNotFound, OperationalError, AssertionError,\
    NotFoundError, ValidationError, NotImplementedError, BaseInternalError, ValidationInternalError,\
    ValidationMultipleError, HTTPError, HTTPDataError, JSONError
from .execution import ExecutionThread, background, insert_work, interval_work,\
    seconds_work, minutes_work, hourly_work, daily_work, weekly_work, monthly_work,\
    seconds_eval, minutes_eval, hourly_eval, daily_eval, weekly_eval, monthly_eval
from .formats import xlsx_to_map
from .httpc import get_f, get, get_json, post_json, put_json, delete_json, HTTPResponse
from .info import NAME, VERSION, AUTHOR, EMAIL, DESCRIPTION, LICENSE, KEYWORDS, URL,\
    COPYRIGHT
from .jsonf import load_json
from .log import MemoryHandler, ThreadFormatter, rotating_handler, smtp_handler,\
    in_signature, has_exception, debug, info, warning, error, critical
from .mail import send_mail, send_mail_a
from .meta import Ordered
from .model import Model, LocalModel, Field, link, operation, view, field
from .mongodb import MongoMap, MongoEncoder
from .observer import Observable
from .structures import OrderedDict, LazyDict, LazyValue, lazy_dict, lazy
from .template import render_template, template_resolve
from .typesf import AbstractType, Type, File, Files, ImageFile, ImageFiles, image, images, Reference,\
    reference, References, references, Encrypted, encrypted, secure
from .unit_test import secured, TestCase
from .util import is_iterable, request_json, get_field, get_object, is_mobile, is_tablet,\
    is_browser, is_bot, browser_info, resolve_alias, page_types, find_types, norm_object, set_object,\
    leafs, load_form, load_locale, get_locale, get_langs, set_locale, reset_locale, anotate_async,\
    run_thread, camel_to_underscore, camel_to_readable, underscore_to_readable, generate_identifier,\
    to_locale, nl_to_br, nl_to_br_jinja, sp_to_nbsp, sp_to_nbsp_jinja, date_time, quote, unquote,\
    is_content_type, parse_content_type, verify, execute, JSONEncoder
from .validation import validate, validate_b, validate_e, safe, eq, gt, gte, lt, lte, not_null,\
    not_empty, not_false, is_in, is_lower, is_simple, is_email, is_url, is_regex, field_eq, field_gt,\
    field_gte, field_lt, field_lte, string_gt, string_lt, string_eq, equals, not_past, not_duplicate,\
    all_different, no_self

from .amazon import get_connection as get_amazon
from .amazon import get_bucket as get_amazon_bucket
from .amazon import clear_bucket as clear_amazon_bucket
from .amazon import get_key as get_amazon_key
from .amazon import exists_key as exists_amazon_key
from .amazon import delete_key as delete_amazon_key
from .amqp import get_connection as get_amqp
from .amqp import properties as properties_amqp
from .execution import insert_work as run_back
from .execution import insert_work as run_background
from .mongodb import get_connection as get_mongo
from .mongodb import reset_connection as reset_mongo
from .mongodb import get_db as get_mongo_db
from .mongodb import drop_db as drop_mongo_db
from .mongodb import object_id as object_id
from .mongodb import dumps as dumps_mongo
from .mongodb import serialize as serialize_mongo
from .mongodb import directions as directions_mongo
from .pusherc import get_pusher as get_pusher
from .redisdb import get_connection as get_redis
from .redisdb import dumps as dumps_redis



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Flask Quorum
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Flask Quorum.
#
# Hive Flask Quorum is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Flask Quorum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Flask Quorum. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import datetime

import quorum

class ValidationTest(quorum.TestCase):

    @quorum.secured
    def test_eq_number(self):
        methods = [quorum.eq("age", 2)]

        object = dict(age = 2)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(age = 2.0)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(age = 3)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertFalse(result)

        object = dict(age = "2")
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertFalse(result)

        object = dict(age = 2.01)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertFalse(result)

    @quorum.secured
    def test_eq_string(self):
        methods = [quorum.eq("name", "John Doe")]

        object = dict(name = "John Doe")
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(name = "john doe")
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertFalse(result)

        object = dict(name = "JohnDoe")
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertFalse(result)

        object = dict(name = 2)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertFalse(result)

        object = dict(name = 2.0)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertFalse(result)

        object = dict(name = True)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertFalse(result)

    @quorum.secured
    def test_gt(self):
        methods = [quorum.gt("age", 2)]

        object = dict(age = 3)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(age = 2.01)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(age = 2)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertFalse(result)

        object = dict(age = "2")
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        if quorum.legacy.PYTHON_3: self.assertFalse(result)
        else: self.assertTrue(result)

        object = dict(age = 1.99)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertFalse(result)

    @quorum.secured
    def test_gte(self):
        methods = [quorum.gte("age", 2)]

        object = dict(age = 3)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(age = 2)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(age = 2.01)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(age = 2.00)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(age = "2")
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        if quorum.legacy.PYTHON_3: self.assertFalse(result)
        else: self.assertTrue(result)

        object = dict(age = 1.99)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertFalse(result)

    @quorum.secured
    def test_lt(self):
        methods = [quorum.lt("age", 2)]

        object = dict(age = 1)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(age = 1.99)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(age = 2)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertFalse(result)

        object = dict(age = "2")
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertFalse(result)

        object = dict(age = 2.01)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertFalse(result)

    @quorum.secured
    def test_lte(self):
        methods = [quorum.lte("age", 2)]

        object = dict(age = 1)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(age = 2)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(age = 1.99)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(age = 2.00)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(age = "2")
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertFalse(result)

        object = dict(age = 2.01)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertFalse(result)

    @quorum.secured
    def test_not_null(self):
        methods = [quorum.not_null("name")]

        object = dict(name = "John Doe")
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(name = 1)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(name = 1.0)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(name = False)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(name = "")
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(name = None)
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertFalse(result)

    @quorum.secured
    def test_not_empty(self):
        methods = [quorum.not_empty("name")]

        object = dict(name = "John Doe")
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(name = [1, 2, 3])
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(name = "")
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertFalse(result)

        object = dict(name = [])
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertFalse(result)

    @quorum.secured
    def test_not_past(self):
        methods = [quorum.not_past("time")]

        object = dict(time = datetime.datetime(2200, 1, 1))
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertTrue(result)

        object = dict(time = datetime.datetime(2001, 1, 1))
        result = quorum.validation.validate_b(
            methods = methods,
            object = object,
            build = False
        )
        self.assertFalse(result)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Flask Quorum
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Flask Quorum.
#
# Hive Flask Quorum is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Flask Quorum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Flask Quorum. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import json

from . import util
from . import legacy
from . import common
from . import typesf
from . import exceptions

try: import pymongo
except: pymongo = None

try: import bson.json_util
except: bson = None

connection = None
""" The global connection object that should persist
the connection relation with the database service """

url = "mongodb://localhost"
""" The global variable containing the URL to be used
for the connection with the service """

database = "master"
""" The global variable containing the value for the
database to be used in the connection with the service """

class MongoMap(object):
    """
    Encapsulates a mongo collection to provide an interface
    that is compatible with the "normal" key value access
    offered by the python dictionary (map).
    """

    collection = None
    """ The collection to be used as the underlying structure
    for the data access """

    key = None
    """ The name of the key to be used for the "default" search
    for value providing """

    def __init__(self, collection, key = "id"):
        self.collection = collection
        self.key = key

    def get(self, value, default = None):
        return self.collection.find_one({self.key : value}) or default

class MongoEncoder(json.JSONEncoder):

    def default(self, obj, **kwargs):
        if not bson: return json.JSONEncoder.default(self, obj, **kwargs)
        if isinstance(obj, bson.objectid.ObjectId): return str(obj)
        if isinstance(obj, legacy.BYTES): return legacy.str(obj, encoding = "utf-8")
        else: return json.JSONEncoder.default(self, obj, **kwargs)

def get_connection():
    return _get_connection(url)

def reset_connection():
    return _reset_connection()

def get_db():
    connection = get_connection()
    result = _pymongo().uri_parser.parse_uri(url)
    _database = result.get("database", None) or database
    db = connection[_database]
    return db

def drop_db():
    db = get_db()
    names = db.collection_names()
    for name in names:
        if name.startswith("system."): continue
        db.drop_collection(name)

def object_id(value):
    return bson.ObjectId(value)

def dumps(*args):
    return json.dumps(default = serialize, *args)

def serialize(obj):
    if isinstance(obj, common.model().Model): return obj.model
    if isinstance(obj, typesf.AbstractType): return obj.json_v()
    return bson.json_util.default(obj)

def directions(all = False):
    return (
        _pymongo().ASCENDING,
        _pymongo().DESCENDING,
        _pymongo().HASHED
    ) if all else (
        _pymongo().ASCENDING,
        _pymongo().DESCENDING
    )

def is_mongo(obj):
    if bson and isinstance(obj, bson.ObjectId): return True
    if bson and isinstance(obj, bson.DBRef): return True
    return False

def is_new():
    return int(_pymongo().version[0]) >= 3 if pymongo else False

def _store_find_and_modify(store, *args, **kwargs):
    if is_new(): store.find_one_and_update(*args, **kwargs)
    else: store.find_and_modify(*args, **kwargs)

def _store_insert(store, *args, **kwargs):
    if is_new(): store.insert_one(*args, **kwargs)
    else: store.insert(*args, **kwargs)

def _store_update(store, *args, **kwargs):
    if is_new(): store.update_one(*args, **kwargs)
    else: store.update(*args, **kwargs)

def _store_remove(store, *args, **kwargs):
    if is_new(): store.delete_many(*args, **kwargs)
    else: store.remove(*args, **kwargs)

def _store_ensure_index(store, *args, **kwargs):
    kwargs["background"] = kwargs.get("background", True)
    if is_new(): store.create_index(*args, **kwargs)
    else: store.ensure_index(*args, **kwargs)

def _store_ensure_index_many(store, *args, **kwargs):
    directions_l = kwargs.pop("directions", None)
    if directions_l == "all": directions_l = directions(all = True)
    elif directions_l == None: directions_l = directions()
    for direction in directions_l:
        _args = list(args)
        _args[0] = [(_args[0], direction)]
        _store_ensure_index(store, *_args, **kwargs)

def _get_connection(url, connect = False):
    global connection
    if pymongo == None: raise exceptions.ModuleNotFound("pymongo")
    if connection: return connection
    if is_new(): connection = _pymongo().MongoClient(url, connect = connect)
    else: connection = _pymongo().Connection(url)
    return connection

def _reset_connection():
    global connection
    if not connection: return
    if is_new(): connection.close()
    else: connection.disconnect()
    connection = None

def _pymongo(verify = True):
    if verify: util.verify(
        not pymongo == None,
        message = "pymongo library not available",
        exception = exceptions.OperationalError
    )
    return pymongo



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Flask Quorum
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Flask Quorum.
#
# Hive Flask Quorum is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Flask Quorum is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Flask Quorum. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import flask
import smtplib

import email.utils
import email.header

import email.mime.multipart
import email.mime.text

from . import config
from . import common
from . import legacy
from . import execution

def send_mail(
    app = None,
    subject = "",
    sender = None,
    receivers = [],
    data = None,
    plain = None,
    rich = None,
    context = {},
    encoding = "utf-8",
    host = None,
    port = None,
    username = None,
    password = None,
    stls = False,
    safe = True
):
    """
    Sends an email message using the provided :rst:dir:`SMTP_HOST`,
    :rst:dir:`SMTP_USER` and :rst:dir:`SMTP_PASSWORD` configurations.

    The email message is sent under the ``alternative`` mime
    type so that both the plain text and the rich text (html)
    parts are sent in the same message.

    The ``plain`` and ``rich`` arguments allow the user to process
    a template with the context provided by the ``context`` map.

    .. warning::

        This is a blocking call and as such the control flow may block
        for more that a second, if you want a non blocking (asynchronous)
        call please use :func:`quorum.send_mail_a`.

    :type app: Application
    :param app: Optional application object to be used for the\
    rendering operation as the main object for flask. In case this\
    value is not provided the global :attr:`quorum.APP` value is\
    used instead (fallback).
    :type subject: String
    :param subject: The mime subject to be sent with this message,\
    note that if this value is not set many spam filters will consider\
    the message as spam.
    :type sender: String
    :param sender: The email (and name) of the sender of the email, in\
    case this value is not set the :rst:dir:`SMTP_USER` variable is\
    used instead.
    :type receivers: List
    :param receivers: The list of receivers (with email and name) for\
    which the email will be sent.
    :type data: String
    :param data: The buffer containing the data to be used for both the\
    plain and the rich text parts in case the template associated arguments\
    are not set or in case the rendering is not successful.
    :type plain: String
    :param plain: Relative path to the plain text template to be used\
    for the rendering of the email, this path must be relative to the\
    templates folder.
    :type rich: String
    :param rich: Relative path to the rich text template to be used\
    for the rendering of the email, this path must be relative to the\
    templates folder.
    :type context: Dictionary
    :param context: The map containing the complete set of variables that\
    are going to be `"exposed"` to the template rendering engine for both\
    the ``plain`` and the ``rich``.
    :type encoding: String
    :param encoding: The text encoding name that is going to be used in the\
    mail that is going to be sent, should be used with care.
    :type host: String
    :param host: The hostname for the connection of the smtp server that\
    is going to be used, this may be wither a domain of an address.
    :type port: int
    :param port: The tcp port number that is going to be used for the client\
    connection with the server.
    :type username: String
    :param username: Username value that is used for the authentication part\
    of the connection with the smtp server.
    :type password: String
    :param password: Secret password to be used as part of the authentication\
    process of the created smtp connection.
    :type stls: bool
    :param stls: If the connection with the target smtp server should be made\
    using a secure mechanism of a plain text one.
    :type safe: bool
    :param safe: If the current email message should be used using a safe\
    strategy, meaning that newline sequences will be made standard.
    """

    # retrieves the reference to the currently loaded/defined application
    # from the "global" common module (dynamic loading)
    app = app or common.base().APP

    # retrieved the default value of the data value so that no part of
    # the email is left empty (required by specification)
    data = data or "This part of the email is empty"

    # tries to retrieve the URL based definition of the SMTP
    # settings so that they may be used for configuration
    url = config.conf("SMTP_URL", None)
    url_p = legacy.urlparse(url) if url else None
    if url_p:
        host_p, port_p, user_p, password_p, stls_p =\
        url_p.hostname,\
        url_p.port,\
        url_p.username,\
        url_p.password,\
        url_p.scheme == "smtps"
    else:
        host_p, port_p, user_p, password_p, stls_p =\
        None, None, None, None, None

    # runs the defaulting operation for host and port definition
    # that haven't been set (should follow SMTP defaults)
    if host_p == None: host_p = "localhost"
    if port_p == None: port_p = 25

    # tries to retrieve the various configuration values that are going
    # to be used for the establishment of the smtp connection, taking
    # into account both the provided parameters and the configuration
    # variables currently defined for the environment
    host = host or config.conf("SMTP_HOST", host_p)
    port = port or config.conf("SMTP_PORT", port_p, cast = int)
    username = username or config.conf("SMTP_USER", user_p)
    password = password or config.conf("SMTP_PASSWORD", password_p)
    stls = password or stls or config.conf("SMTP_STARTTLS", stls_p, cast = int)

    # sets the sender with the smtp user value in case no values
    # has been provided (expected behavior)
    sender = sender or username

    # renders the (possible existing) templates in both the plain
    # and rich text object retrieving the final data to be sent
    plain_data = plain and _render(app, plain, **context) or data
    html_data = rich and _render(app, rich, **context) or data

    # verifies the existence of data (both plain and HTML) and in
    # there's data it should be encoded using the currently provided
    # one so that the raw data is used instead of the unicode one
    if plain_data: plain_data = plain_data.encode(encoding)
    if html_data: html_data = html_data.encode(encoding)

    # creates the mime's multipart object with the appropriate header
    # values set and in the alternative model (for HTML compatibility)
    message = _multipart()
    message["Subject"] = subject
    message["From"] = _format(sender)
    message["To"] = ", ".join(_format(receiver) for receiver in receivers)

    # creates both the plain text and the rich text (html) objects
    # from the provided data and then attached them to the message
    # (multipart alternative) that is the base structure
    plain = plain_data and _plain(plain_data, encoding = encoding)
    html = html_data and _html(html_data, encoding = encoding)
    plain and message.attach(plain)
    html and message.attach(html)

    # converts the created message into a plain string value and then
    # in case the safe flag is active replaces the proper newline
    # sequences so that the mail string is made standard
    contents = message.as_string()
    if safe:
        contents = contents.replace("\r\n", "\n")
        contents = contents.replace("\n", "\r\n")

    # creates the connection with the smtp server and starts the tls
    # connection to send the created email message
    server = smtplib.SMTP(host, port = port)
    try:
        if stls: server.starttls()
        server.login(username, password)
        server.sendmail(sender, receivers, contents)
    finally:
        server.quit()

def send_mail_a(*args, **kwargs):
    """
    Asynchronous call to the :func:`quorum.send_mail` function that
    is executed in a different thread from the current one. The currently
    loaded queue system is used for the sending of the email, for more
    information check on :func:`quorum.run_background` .

    .. note::

        The arguments to be send for this function are the same as the one
        present in the original :func:`quorum.send_mail` function.
    """

    execution.insert_work(send_mail, args, kwargs)

def _multipart():
    return email.mime.multipart.MIMEMultipart("alternative")

def _plain(contents, encoding = "utf-8"):
    return email.mime.text.MIMEText(contents, "plain", encoding)

def _html(contents, encoding = "utf-8"):
    return email.mime.text.MIMEText(contents, "html", encoding)

def _format(address, encoding = "utf-8"):
    address_name, address_email = email.utils.parseaddr(address)
    if legacy.is_bytes(address_name): address_name = address_name.decode(encoding)
    address_name = email.header.Header(address_name, charset = encoding)
    address_name = address_name.encode()
    address_email = str(address_email)
    return "%s <%s>" % (address_name, address_email)

def _render(app, template_name, **context):
    template = app.jinja_env.get_or_select_template(template_name)
    return flask.templating._render(template, context, app)
